//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCADAutoCal.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS 
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function  
//				dscADAutoCal(). This sample code describes the usage
//				for Hercules II EBX SBC. AutoCalibration will use the onboard
//              hardware to calibrate the A/D circuitry on the board,
//              then verify that the process results fall within tolerances.
//              The calibration values will be stored in the EEPROM,
//              along with one set of values to be loaded at boot time.
// ************************* History ********************
// Version		Engineer		Date		Description		
// 1.0			James Moore		07/31/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif
#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

#endif

#define OFFSET_LSB_TOLERANCE 2
#define GAIN_LSB_TOLERANCE 2

// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCB dsccb;  // structure containing board settings
WORD sample;  // sample reading
DSCAUTOCAL dscautocal; // structure containing auto-calibration settings
ERRPARAMS errorParams; // structure for returning error messages
int intBuff;	       // integer variable
float floatBuff;       // float variable
int i;	               // miscellaneous counter

char input_buffer[20]; // Buffer for parsing user character input.

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. AutoCal Settings Initialization
//	    IV. Auto-Calibration
//		 V. Calibration Verification
//	    VI. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if ( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the HERCULES board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf ( "\nHERCULES BOARD INITIALIZATION:\n" );

	dsccb.boardtype = DSC_HERCEBX;

    // The Hercules II has a fixed base address.
    dsccb.base_address = 0x240;

	dsccb.int_level = 5;

	if ( dscInitBoard ( dsccb.boardtype, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}


	//=========================================================================
	// III. AUTOCAL SETTINGS INITIALIZATION
	//
	//		Initialize the structure containing the AD autocal settings. 
	//
	//=========================================================================

	do
    {

		printf ( "\n\nAUTOCAL SETTINGS INITIALIZATION:\n" );

        printf ( "\nRange ; meaning\n0:\t10V\tbipolar\n1:\t5V\tbipolar\n2:\t2.5V\tbipolar\n3:\t1.25V\tbipolar\n" );
        printf ( "4:\t10V\tunipolar\n5:\t5V\tunipolar\n6:\t2.5V\tunipolar\n7:\t1.25V\tunipolar\n" );

		printf ( "Enter the range to calibrate (0-7)\nor 255 to calibrate all ranges (default: 255): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscautocal.adrange = 255;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    	    dscautocal.adrange = (BYTE)intBuff;
	    }

		printf ( "Enter the AD boot range (0-7; default: 0): " );

	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    		dscautocal.boot_adrange = 0;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    		dscautocal.boot_adrange = (BYTE) intBuff;
	    }

	//=========================================================================
	// IV. AUTO-CALIBRATION
	//
	//	   Perform the actual AD auto-calibration of the board.
	//
	//=========================================================================

		printf ( "\nAUTO-CALIBRATION:\n" );
		printf ( "Starting AD auto-calibration...This will take a moment\n" );

		if ( dscADAutoCal ( dscb, &dscautocal ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf ( stderr, "dscADAutoCal error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
			return 0;
		}

		printf ( "Completed\n\n" );

	//=========================================================================
	// V. CALIBRATION VERIFICATION
	//
	//	  To verify that the auto-calibration was successfully completed, you
	//	  should run dscADCalVerify() on the 8 possible configuration modes to
	//	  ensure that the gain and offset errors meet a specified tolerance (2
	//	  in this example). You can obtain the possible combinations by writing
	//	  a for loop and bit-wise operations similar to those used below.
	//
	//=========================================================================

		printf ( "\nCALIBRATION VERIFICATION:\n" );

		for ( i = 0; i < 8; i++ )
		{
			dscautocal.adrange = i;
			if ( ( result = dscADCalVerify ( dscb, &dscautocal ) ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscADCalVerify error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
				return 0;
			}

			printf ( "Configuration Mode: %d, Offset Error: %10.3f, Gain Error: %10.3f\n", i, dscautocal.ad_offset, dscautocal.ad_gain ); 

			if ( fabs( dscautocal.ad_offset ) > OFFSET_LSB_TOLERANCE || 
				 fabs( dscautocal.ad_gain ) > GAIN_LSB_TOLERANCE )
				printf ( "Value for offset or gain exceeded specified tolerance\n" );
			else printf ( "Values for offset and gain met specified tolerance\n" );
		}

		printf ("\nEnter 'q' to exit or any key to continue : " );

	    fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q');


	//=========================================================================
	// VI. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf ( "\nDSCADAutoCal completed. \n" );
	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
