//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCADSampleInt.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS 
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function  
//				dscADSampleInt(). This sample code describes the usage
//				for Hercules II EBX SBC. The driver will use the incumbent OS
//              in conjunction with the interrupt selected to transfer many
//              values from the A/D converter to user space, and once stopped,
//              will display a single set of those channel values,
//              interpretting them based on the mode given by the user.
// ************************* History ********************
// Version		Engineer		Date		Description		
// 1.0			James Moore		07/27/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif

#ifdef _WIN32_WCE
#include <string.h>

static int kbhit()
{
	int i;
	int result=0;

	
	result |= GetAsyncKeyState(VK_RETURN);
	result |= GetAsyncKeyState(VK_SPACE);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	//number keys, 0-9
	for (i=48; i<=57; i++)
		result |= GetAsyncKeyState(i);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	//capital character keys, A-Z
	for (i=65; i<=90; i++)
		result |= GetAsyncKeyState(i);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	//lower case keys, a-z
	for (i=97; i<=122; i++)
		result |= GetAsyncKeyState(i);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	return result;
}

#endif


// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
#include <mem.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif

// macros defined
#define SLEEP_TIME 1000
#define HERCULESII_MAX_AD_CHANNEL_NUMBER 31

// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCB dsccb;  // structure containing board settings
DSCADSETTINGS dscadsettings; // structure containing A/D conversion settings
DSCAIOINT dscaioint;         // structure containing auto-calibration settings
ERRPARAMS errorParams;       // structure for returning error messages
DSCS dscs;        // used for interrupts.
DFLOAT voltage;   // voltage value
int intBuff;      // integer variable
long longBuff;	  // long variable
float floatBuff;  // float variable
int i;	          // miscellaneous counter
int k;            // channel counter

char input_buffer[20]; // Buffer for parsing user character input.

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. AD Settings
//	    IV. Interrupt Settings
//		 V. Perform Interrupts
//	    VI. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if ( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the HERCULES board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf ( "\nHERCULES BOARD INITIALIZATION:\n" );

	dsccb.boardtype = DSC_HERCEBX;

    // The Hercules II has a fixed base address.
    dsccb.base_address = 0x240;

	printf ( "Enter the DAQ interrupt level (default: 5): " );
   	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dsccb.int_level = 5;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dsccb.int_level = (BYTE) intBuff;
	}

	if ( dscInitBoard ( dsccb.boardtype, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// III. AD SETTINGS INITIALIZATION
	//
	//	    Initialize the structure containing the AD conversion settings and
	//		then pass it to the driver.
	//
	//=========================================================================

	printf ( "\nAD SETTINGS INITIALIZATION\n" );

	memset(&dscadsettings, 0, sizeof(DSCADSETTINGS));

	dscadsettings.range = RANGE_10;

	printf ( "Enter the polarity (0 for BIPOLAR, 1 for UNIPOLAR; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
	    dscadsettings.polarity = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.polarity = (BYTE) intBuff;
	}

	printf ( "Enter the gain\n(0 for GAIN 1, 1 for GAIN 2, 2 for GAIN 4, 3 for GAIN 8; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.gain = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.gain = (BYTE) intBuff;
	}

   	// This feature will load the A/D calibration settings from the EEPROM

	printf ( "Enter the load calibration flag (0 for FALSE, 1 for TRUE; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.load_cal = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.load_cal = (BYTE) intBuff;
	}

	printf ( "Enter single/differential (0 is single-ended, 1 is differential; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.addiff = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.addiff = (BYTE) intBuff;
	}

	dscadsettings.current_channel = 0;

	dscadsettings.scan_interval = SCAN_INTERVAL_9;

	if ( dscADSetSettings ( dscb, &dscadsettings ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}

	//=========================================================================
	// IV. I/O INTERRUPT SETTINGS INITIALIZATION
	//
	//	   Initialize the structure containing the analog I/O interrupt
	//	   settings.
	//
	//	   NOTE: You must allocate space for the buffer holding the returned
	//		     sample values. Also, be generous in allocating storage.
	//			 Allocating insufficient memory to hold sample data will result
	//			 in improper behavior of the driver, such as hanging interrupt
	//			 operations or assertion errors.
	//
	//=========================================================================

	printf ( "\nI/O INTERRUPT SETTINGS INITIALIZATION\n" );

	memset(&dscaioint, 0, sizeof(DSCAIOINT));

	printf ( "Enter the number of conversions\n(must be a multiple of FIFO depth; default: 1024) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.num_conversions = 1024;
	}
	else
	{
        sscanf ( input_buffer, "%ld", &longBuff );
    	dscaioint.num_conversions = (DWORD) longBuff;
	}

	printf ( "Enter the conversion rate in Hz\n(must be less than 250000; default: 100000) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.conversion_rate = 100000;
	}
	else
	{
        sscanf ( input_buffer, "%f", &floatBuff );
    	dscaioint.conversion_rate = (FLOAT) floatBuff;
	}

	printf ( "Enter the cycle flag (0 for FALSE, 1 for TRUE; default: 0) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.cycle = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscaioint.cycle = (BOOL) intBuff;
	}

	printf ( "Enter the internal clock flag (0 for FALSE, 1 for TRUE; default: 1) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.internal_clock  = 1;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscaioint.internal_clock  = (BOOL) intBuff;
	}

	printf ( "Enter the lowest channel in the scan range (0-%d; default: 0) : ", HERCULESII_MAX_AD_CHANNEL_NUMBER );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.low_channel = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscaioint.low_channel = (BYTE) intBuff;
	}


	printf ( "Enter the highest channel in the scan range (0-%d; default: %d) : ", HERCULESII_MAX_AD_CHANNEL_NUMBER, HERCULESII_MAX_AD_CHANNEL_NUMBER );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.high_channel = HERCULESII_MAX_AD_CHANNEL_NUMBER;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscaioint.high_channel = (BYTE) intBuff;
	}

	dscaioint.external_gate_enable = 0; // can enable it if need be

	dscaioint.internal_clock_gate = 0;   // can enable it if need be

	dscaioint.fifo_enab = 1;

	printf ( "Enter the FIFO usage depth\n(must be less than 2048 and greater than 0: default: 1024) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.fifo_depth = 1024;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscaioint.fifo_depth = (WORD) intBuff;
	}

	// Dump Threshold here is the threshold at which the data from
	//   kernel space is copied to the structure submitted in user space.

	printf ( "Enter the transfer to user memory threshold\n( must be less or equal to num conversions; default: 1024 ) : " );
    fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.dump_threshold = 1024;
	}
	else
	{
        sscanf ( input_buffer, "%ld", &longBuff );
    	dscaioint.dump_threshold = (DWORD)longBuff;
	}

    dscaioint.sample_values = (DSCSAMPLE*)malloc( sizeof(DSCSAMPLE) * dscaioint.num_conversions);

	//=========================================================================
	// V. SAMPLING AND OUTPUT
	//
	//    Perform the actual sampling and then output the results. To calculate
	//	  the actual input voltages, we must convert the sample code (which
	//	  must be cast to a short to get the correct code) and then plug it
	//	  into one of the formulas located in the manual for your board (under
	//	  "A/D Conversion Formulas"). 
	//=========================================================================

	printf ( "\nSAMPLING AND OUTPUT\n" );

	do 	{
		dscs.transfers = 0;
		dscs.overflows = 0;
		dscs.op_type = OP_TYPE_INT;

		if ( dscADSampleInt( dscb, &dscaioint ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf ( stderr, "dscADSampleInt error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
			free( dscaioint.sample_values ); // remember to deallocate malloc() memory
			return 0;
		}

		do
        {
			dscSleep(SLEEP_TIME);
			dscGetStatus(dscb, &dscs);
			printf ( "Fifo Overflows %lu, A/D Int Trans in progress: %lu, Total Trans %lu\n", dscs.overflows, dscs.transfers, dscs.total_transfers);
        } while ( dscs.op_type != OP_TYPE_NONE && !kbhit() );
	
		// cancel interrupts manually for recycled mode or if interrupts are still running
		if ( dscs.op_type != OP_TYPE_NONE )
		{
			if ( ( result = dscCancelOp(dscb)) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscCancelOp error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
				free( dscaioint.sample_values ); // remember to deallocate malloc() memory
				return 0;
			}
		}

		printf ( "\nChannel  A/D Code    Voltage\n" );

		for ( i = 0, k = dscaioint.low_channel; i < dscaioint.high_channel - dscaioint.low_channel + 1; i += 1, k += 1 )
		{
			if ( dscadsettings.polarity )
				printf ( "%2d\t%6.3hd\t", k, dscaioint.sample_values[i] + 32768 );
			else
				printf ( "%2d\t%6.3hd\t", k, dscaioint.sample_values[i] );

            if ( dscADCodeToVoltage ( dscb, dscadsettings, dscaioint.sample_values[i], &voltage ) != DE_NONE)
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscADCodeToVoltage error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				free ( dscaioint.sample_values );
				return 0;
			}

			printf ( "%6.3lfV\n", voltage );
		}
		
		printf ( "\n\n" );
		
		printf ( "Enter 'q' to quit or any key to continue " );

	    fgets ( input_buffer, 20, stdin );


    } while ( input_buffer[0] != 'q' );

	//=========================================================================
	// VI. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the memory allocated for sample values
	//	   2. free the driver resources
	//=========================================================================

	free( dscaioint.sample_values );

	dscFree();

	printf ( "\nDSCADSampleInt completed.\n" );

	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	printf ( "For users running cycled mode: to EXIT, hold down the 'q' key\n" );
	printf ( "on the keyboard then press 'Enter' when 'q' letters appear on screen\n\n" );
	main();

	return 0;
}

#endif
