//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCADScan.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS 
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function  
//				dscADScan(). This sample code describes the usage
//				for Hercules II EBX SBC. Hardware on the board will be commanded
//              to scan the A/D channels passed, and return them to the
//              sample array. Then the values will be interpretted based on
//              the mode supplied by the user.
//              Each A/D hardware scan consists of a sweep of all the 
//              selected channels at a time, rather than only one channel.
// ************************* History ********************
// Version		Engineer		Date		Description		
// 1.0			James Moore		08/06/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
#include <mem.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

#endif

// macro defines 

#define HERCULESII_MAX_AD_CHANNEL_NUMBER 31

// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCB dsccb;  // structure containing board settings
DSCADSETTINGS dscadsettings; // structure containing A/D conversion settings
DSCADSCAN dscadscan;   // structure containing A/D scan settings
DSCSAMPLE* samples;    // sample readings
DFLOAT voltage;        // actual voltage
ERRPARAMS errorParams; // structure for returning error messages
int intBuff;           // integer variable
long longBuff;	       // long variable
float floatBuff;       // float variable

int i;        // miscellaneous counter
int k;        // channel counter

char input_buffer[20]; // Buffer for parsing user character input.

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//     STEPS TO FOLLOW:
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. AD Settings Initialization
//		IV. AD Scan Initialization
//	     V. Scanning and Output
//	    VI. Cleanup
//
//=============================================================================

int main( void )
{

	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if ( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the HERCULES board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf ( "\nHERCULES BOARD INITIALIZATION:\n" );

	dsccb.boardtype = DSC_HERCEBX;
	
    // The Hercules II has a fixed base address.
    dsccb.base_address = 0x240;

	dsccb.int_level = 5;

	if ( dscInitBoard ( dsccb.boardtype, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// III. AD SETTINGS INITIALIZATION
	//
	//	    Initialize the structure containing the AD conversion settings and
	//		then pass it to the driver.
	//
	//=========================================================================

	printf ( "\nAD SETTINGS INITIALIZATION\n" );

	memset(&dscadsettings, 0, sizeof(DSCADSETTINGS));

	dscadsettings.range = RANGE_10;

	printf ( "Enter the polarity (0 for BIPOLAR, 1 for UNIPOLAR; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.polarity = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.polarity = (BYTE) intBuff;
	}

	printf ( "Enter the gain\n(0 for GAIN 1, 1 for GAIN 2, 2 for GAIN 4, 3 for GAIN 8; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
   		dscadsettings.gain = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.gain = (BYTE) intBuff;
	}

    // This feature loads the A/D TrimDAC calibration values from the EEPROM locations
    //   associated with the mode (voltage gain and polarity) the user specified.

   	printf ( "Load corresponding A/D Calibration settings? (0: no,1: yes; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.load_cal = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.load_cal = intBuff;
	}

	printf ( "Enter single/differential\n(0 is single-ended, 1 is differential; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.addiff = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.addiff = (BYTE) intBuff;
	}

	dscadsettings.current_channel = 0;

	dscadsettings.scan_interval = SCAN_INTERVAL_9; // 9 microsec scan interval

	if ( dscADSetSettings( dscb, &dscadsettings ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// IV. AD SCAN INITIALIZATION
	//
	//	   Initialize the structure containing the AD scan setting and allocate
	//	   memory for our buffer containing sample values.
	//
	//=========================================================================

	printf ( "\nAD SCAN INITIALIZATION\n" );

	printf ( "Enter the lowest channel in the scan range (0-%d; default: 0) : ", HERCULESII_MAX_AD_CHANNEL_NUMBER );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadscan.low_channel = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadscan.low_channel = (BYTE) intBuff;
	}


	printf ( "Enter the highest channel in the scan range (0-%d; default: %d) : ", HERCULESII_MAX_AD_CHANNEL_NUMBER, HERCULESII_MAX_AD_CHANNEL_NUMBER );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadscan.high_channel = HERCULESII_MAX_AD_CHANNEL_NUMBER;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadscan.high_channel = (BYTE) intBuff;
	}

    dscadscan.gain = dscadsettings.gain;

	samples = (DSCSAMPLE*)malloc( sizeof(DSCSAMPLE) * ( dscadscan.high_channel - dscadscan.low_channel + 1 ) );

	if ( samples == NULL ) {
		fprintf ( stderr, "Unable to allocate memory\n" );
		return 0;
	}

	//=========================================================================
	// V. SCANNING AND OUTPUT
	//
	//    Perform the actual sampling and then output the results. To calculate
	//	  the actual input voltages, we must convert the sample code (which
	//	  must be cast to a short to get the correct code) and then plug it
	//	  into one of the formulas located in the manual for your board (under
	//	  "A/D Conversion Formulas"). 
	//=========================================================================

	printf ( "\nSCANNING AND OUTPUT\n" );

	do 
	{
		if ( dscADScan( dscb, &dscadscan, samples ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf ( stderr, "dscADScan error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
			free( samples ); // remember to deallocate malloc() memory
			return 0;
		}

		printf ( "\nChannel  A/D Code    Voltage\n" );

		for( i = 0, k = dscadscan.low_channel; i < dscadscan.high_channel - dscadscan.low_channel + 1; i += 1, k += 1 )
		{
			if ( dscadsettings.polarity )
				printf ( "%2d\t%6.3hd\t", k, dscadscan.sample_values[i] + 32768 );
			else
				printf ( "%2d\t%6.3hd\t", k, dscadscan.sample_values[i] );

            if ( dscADCodeToVoltage ( dscb, dscadsettings, dscadscan.sample_values[i], &voltage ) != DE_NONE)
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscADCodeToVoltage error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				free(samples);
				return 0;
			}

			printf ( "%6.3lfV\n", voltage );
		}

		printf ( "\nEnter 'q' to quit or any key to continue " );

    	fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );

	//=========================================================================
	// VI. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the memory allocated for sample values
	//	   2. free the driver resources
	//=========================================================================

	free( samples );
	
	dscFree();

	printf ( "\nDSCADScan completed.\n" );

	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
