//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCUserInt.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS 
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function  
//				dscUserInt(). This sample code describes the usage
//				for Hercues II EBX SBC. Function 'count' is called each time
//              a timer raises an interrupt.
// ************************* History ********************
// Version		Engineer		Date		Description		
// 1.0			James Moore		07/30/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif


// macros defined
#define SLEEP_TIME 1000

// var declarations
BYTE result;   // returned error code
DSCB dscb;     // handle used to refer to the board
DSCCB dsccb;   // structure containing board settings
ERRPARAMS errorParams;    // spits out errors
DSCUSERINT dscuserint;    // userint setup structure
BYTE byteBuff;    // byte variable
int intBuff;      // integer variable
long longBuff;	  // long variable
float floatBuff;  // float variable

int i;          // miscelaneous counter

char input_buffer[20]; // Buffer for parsing user character input.

int counter;	// variable to be modified by user defined function

void count(void* params);  // user defined function

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. User interrupt setup
//	    IV. Trigger the interrupts (just for demo purposes)
//	     V. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//	  Initialize the driver, using the driver version for validation
	//
	//=========================================================================

	if ( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the HERCULES board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf ( "\nHERCULES BOARD INITIALIZATION:\n" );

	dsccb.boardtype = DSC_HERCEBX;

    // The Hercules II has a fixed base address.
    dsccb.base_address = 0x240;

	printf ( "Enter the DAQ interrupt level in decimal (default: 5): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dsccb.int_level = 5;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dsccb.int_level = (BYTE) intBuff;
	}

	if ( dscInitBoard ( dsccb.boardtype, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// III. USER INTERRUPT SETUP
	//
	//		Sets up the following parameters for the user interrupt:
	//			user interrupt function
	//			interrupt source
	//			clock source (if the interrupt source is a counter/timer)
	//			interrupt rate (if the interrupt source is a counter/timer
	//	 						and the clock source is internal)
	//
	//	Note:	The user can also use dscCounterDirectSet() to set the
	//			counter rate and then set rate = 0. The user is forced to use
	//			dscCounterDirectSet() if rate = 0
	//
	//=========================================================================

	// if using dscCounterDirectSet to specify the user rate then set
	// dscuserint.rate = 0.
	// For dscCounterDirectSet the parameters are specified as below :	
	// "BYTE dscCounterDirectSet( DSCCB, BYTE mode, WORD nDivider, BYTE counter)";
	//
	//	 dscCounterDirectSet(dscb, 0x34, 0, 0);		
	//

	counter = 0;			// set the counter to 0

	// On HERCULES, you have 1 choices for the interrupt source
	//					     1 choice for the counter
	//					     3 choice for the clksource :
	//
	//	intsource		counter		clksource		description
	//	0				0			0				counter1, 100KHz clock
	//	0				0			1				counter1, 10MHz clock
	//	0				0			2				counter1, external clock source
	//  1               0           0               DIO interrupt.
	
	dscuserint.intsource = 0;
	dscuserint.counter = 0;
	dscuserint.rate = 1.1f;


	printf ( "Enter the interrupt source (0 internal, 1 external; default: 0) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscuserint.intsource = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscuserint.intsource = (BYTE) intBuff;
	}

	if (dscuserint.intsource == 0)
	{
		printf ( "Enter the clock source (0 for 100KHz, 1 for 10MHz, 2 for external clock; default: 0) : " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
		    dscuserint.clksource = 0;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
		    dscuserint.clksource = (BYTE) intBuff;
	    }

		printf ( "Enter the interrupt rate in hertz (default: 1000): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    		dscuserint.rate = 1000;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    		dscuserint.rate = (FLOAT) floatBuff;
	    }
	}
	else
		dscuserint.rate = 0;

	if ( dscUserInt ( dscb, &dscuserint, count ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscUserInt error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}


	// =================================================================================
	// If dscuserint.intsource == 0 and dscuserint.clksource == 2 
	// call the following to program counter timer 1
	//
    // dscCounterDirectSet(dscb, 0x82, 2, 1);
	//
	//=================================================================================


	//================================================================
	// This section is if dscuserint.intsource == 1 or External source.
	// Interrupt is generated via DIO interrupts. Use J8, pin 39, 
	// DIOLATCH to generate DIO interrupts. 
	//================================================================

	if ( dscuserint.intsource == USER_INT_SOURCE_EXTERNAL )
	{
		dscInp ( dsccb.base_address + 12, &byteBuff );
		byteBuff |= 0x20;  //Set DIOCTR1 to 1
		dscOutp ( dsccb.base_address + 12, byteBuff );
	}
   
	//===========================================================================
	//	IV. CASES FOR USERINT 
	//
	//      Tests userinterrupt
	//===========================================================================

	while ( counter < 1024 )
	{
		printf ( "UserInt counter %d\n", counter );
		dscSleep(SLEEP_TIME);
	}

	// cancel interrupts manually if interrupts are still running
	if ( dscCancelOp(dscb) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscCancelOp error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// V. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//=========================================================================

	if ( dscClearUserInterruptFunction(dscb) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscClearUserInterruptFunction error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}

	dscFree();

	printf ( "\nDSCUserInt completed.\n" );

	return 0;
} // end main()

//=============================================================================
//	The user interrupt routine
//
//	Note: If using DOS, you cannot execute dsc driver functions to access the board
//	both in the main program and in the user interrupt routine. This is because
//	the driver locks the board in the main program when it is being used so that
//	no other program can access it and interfere with a function while it is executing.
//	To access the board inside the interrupt routine in DOS, use direct I/O functions
//	( inp(addr), outp(addr,data) ).
//
//=============================================================================

void count(void* params)
{
	counter ++;
}



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
