//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MeasureReferences.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS 
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD functions  
//				dscGetReferenceVoltages(), dscDAGetOffsets(), dscSetCalMux(),
//              dscSetReferenceVoltages(), dscDASetOffsets().
//              This sample code describes the usage for Hercules II EBX SBC.
//              The user places Volt-meter leads on header pins for precision
//              voltage measurements, selects measure references, and enters
//              values seen on Volt-meter for each.
// ************************* History ********************
// Version		Engineer		Date		Description		
// 1.0			James Moore		07/31/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

#endif

// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
DSCAUTOCAL dscautocal; // structure containing auto-calibration settings
ERRPARAMS errorParams; // errorParams
int menu;
float floatBuff;

int i;	     // miscellaneous counter

char input_buffer[20]; // Buffer for parsing user character input.

DFLOAT refs[5];

DFLOAT default_refs[] = { 0.0002, 0.0025, 1.11529, 4.77379 };

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. Measure References
//	    IV. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if ( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the HERCULES board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf ( "\nHERCULES BOARD INITIALIZATION:\n" );

	dsccb.boardtype = DSC_HERCEBX;

    // The Hercules II has a fixed base address.
    dsccb.base_address = 0x240;

	dsccb.int_level = 5;

	if ( dscInitBoard ( dsccb.boardtype, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	printf ( "\n" );

	//=========================================================================
	// III. MEASURE REFERENCES
	//
	//		Measure the reference voltages for your board and then mention how
	//		to store these values in the EEPROM. 
	//
	//=========================================================================

	do {
		printf ( "\nSelect from the following menu \n" );
		printf ( "0) Exit\n" );
		printf ( "1) EEPROM dump \n" );
		printf ( "2) Measure A/D reference voltages\n" );
		printf ( "3) Measure D/A offset voltages\n" );

        fgets ( input_buffer, 20, stdin );
    	sscanf ( input_buffer, "%d", &menu );

		if ( menu == 0 ) 
			break;
		else if ( menu == 1 )
		{
			if ( dscGetReferenceVoltages ( dscb, refs ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscGetReferenceVoltages error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
				return 0;
			}
	
			printf ( "\nThe stored A/D Reference Voltages are:\n" );
			for ( i = 0; i < 4 ; i++ )
			{
				 printf ( "%d: %9.5lfV\n", i, refs[i] );
			}

			printf ( "\n" );

			if ( dscDAGetOffsets ( dscb, refs, 1 ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscDAGetOffsets error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}

			printf ( "The stored D/A average offset voltage is: %9.5lfV\n", refs[0] );

		}
		else if ( menu == 2 )
		{
		
			printf ( "\nPlace the ground probe of your voltmeter\non pin 2 of J33 and the positive probe \non pin 1 of J33\n\n" );
	
			if ( dscSetCalMux (dscb, 1) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscSetCalMux error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}

			for ( i = 0; i < 4; i++ )
			{
				if ( dscADSetChannel ( dscb, (unsigned char) i, (unsigned char)i ) != DE_NONE )
				{
					dscGetLastError(&errorParams);
					fprintf( stderr, "dscADSetChannel error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
					return 0;
				}
		
				printf ( "Enter the voltage on the voltmeter\n(cal mux channel %d; default approximates: %9.5lfV): ", i, default_refs[i] );
	            fgets ( input_buffer, 20, stdin );
	            if ( input_buffer[0] == '\n' )
	            {
    	            refs[i] = default_refs[i];
	            }
	            else
	            {
                    sscanf ( input_buffer, "%f", &floatBuff );
                    refs[i] = floatBuff;
	            }

			}

			if ( dscSetCalMux (dscb, 0) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscSetCalMux error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}

			if ( dscSetReferenceVoltages ( dscb, refs) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscSetReferenceVoltages error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}

			for ( i = 0; i < 4; i++ )
				refs[i] = 0;
	
			if ( dscGetReferenceVoltages ( dscb, refs ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscGetReferenceVoltages error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
				return 0;
			}

			printf ( "The stored A/D Reference Voltages are:\n" );
			for ( i = 0; i < 4 ; i++)
			{
				 printf ( "%d: %9.5lfV\n", i, refs[i]);
			}

			printf ( "\n" );

			printf ( "Complete!\n" );
		}
		else if ( menu == 3 )
		{
			DFLOAT average;

			printf ( "\nPlace the ground probe of your voltmeter on pin 5 of J9.\n" ); 
			dscSetCalMux ( dscb, 0 );
			
			dscRegisterWrite ( dscb, 1, 0x04 );
			for ( i = 0; i < 4; i++ )
            {
				if ( dscDAConvert ( dscb, (BYTE)i, 0) != DE_NONE )
				{
					dscGetLastError(&errorParams);
					fprintf( stderr, "dscDAConvert error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
					return 0;
				}
			}

			average = 0;
			
			for ( i = 0; i < 4; i++ )
			{
				printf ( "Place the positive probe on pin %d of J9. Enter the voltmeter reading: ", i+1 );
	            fgets ( input_buffer, 20, stdin );
                sscanf( input_buffer, "%lf", &refs[i] );

				average += (refs[i] - refs[0]);
			}

			average /= 4.0;

			printf ( "\nNow storing the average D/A offset [%1.5lfV] into the EEPROM..\n", average );
			
			refs[0] = average;
			
			if ( dscDASetOffsets(dscb, refs, 1) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscDASetOffsets error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
				return 0;
			}

			for ( i =0; i < 4; i++)
				refs[i] =0;
	
			if ( dscDAGetOffsets ( dscb, refs, 1 ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscDAGetOffsets error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
				return 0;
			}
	

			printf ( "\nThe stored D/A average offset voltage is: %9.5lfV\n", refs[0] );

			printf ( "\n" );

			printf ( "Complete!\n" );

		}
	} while (1);
	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf ( "\nMeasureReferences completed. \n" );

	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
